@@ -1,10 +1,8 @@
-from ast import Bytes, parse
 from codecs import decode
 from http.client import CONTINUE, OK
 from io import BufferedIOBase, BufferedReader, IOBase,BytesIO
 from os import system
 import pywebhost
-from pywebhost.adapter.websocket import Websocket
 import types
 from ..handler import Request
 from http import HTTPStatus
@@ -235,40 +233,6 @@ def WriteContentToRequest(
             if send_range(request):return True
     return send_once(request)
 
-@ModuleWrapper
-def WebsocketSessionWrapper(raw_frames=False):
-    '''Wrapper for websocket requests
-
-    Usage:
-
-        ...
-        from pywebhost import Websocket
-        class WSApp(Websocket):
-            def onOpen(self):
-                ...
-            def onClose(self):
-                ...
-            def onRecevie(self):
-                ...
-        @server.route('/ws')
-        @WebsocketSessionWrapper(raw_frames=False)
-        def wsapp(request : Request,content):
-            return WSApp
-    Args:
-
-        raw_frames (bool, optional): To receive concatnated content as bytearrary or raw websocket frames. Defaults to False.
-    '''
-    def suffix(request,function_result : pywebhost.Websocket):
-        if not function_result:
-            return # Dropping connection
-        if not issubclass(function_result,Websocket):
-            raise TypeError('A `pywebhost.Websocket` (sub) class is required') 
-        # Making the websocket connection
-        session = function_result(request,raw_frames=raw_frames)
-        session.handshake()
-        session.serve()
-    return None , suffix
-
 @ModuleWrapper
 def VerbRestrictionWrapper(verbs : list = ['GET','POST']) -> None:
     '''Wrapper to limit request to a certain range of verbs
@@ -377,8 +341,9 @@ def Base64MessageWrapper(decode=True,encode=True,read=True,write=True):
         binary = any2str(readstream(request)) if previous_prefix_result is None else previous_prefix_result
         return binary if not decode else base64.b64decode(binary)
     def suffix(request,function_result):
-        if function_result and encode:
-            binary = base64.b64encode(function_result.encode())
+        if function_result:
+            binary = function_result.encode()
+            if encode:base64.b64encode(binary)
             if write:writestream(request,binary)
             return binary
         elif function_result:
